<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * This script handles the session report generation in batch task for a single user.
 * It will produce a single PDF report that is pushed immediately to output
 * for downloading by a batch agent. No file is stored into the system.
 * userid must be provided.
 * This script should be sheduled in a CURL call stack or a multi_CURL parallel call.
 *
 * @package    report_trainingsessions
 * @category   report
 * @author     Valery Fremaux (valery.fremaux@gmail.com)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require('../../../../config.php');

ob_start();
require_once($CFG->dirroot.'/blocks/use_stats/locallib.php');
require_once($CFG->dirroot.'/blocks/use_stats/xlib.php');
require_once($CFG->dirroot.'/report/trainingsessions/locallib.php');
require_once($CFG->dirroot.'/report/trainingsessions/pro/renderers/pdfrenderers.php');

$id = required_param('id', PARAM_INT); // The course id (context for user targets)
$userid = required_param('userid', PARAM_INT); // User id.
$filename = optional_param('outputname', '', PARAM_FILE);

ini_set('memory_limit', '512M');

if (!$course = $DB->get_record('course', array('id' => $id))) {
    // Do NOT print_error here as we are a document writer.
    die ('Invalid course ID');
}
$context = context_course::instance($course->id);

if (!$user = $DB->get_record('user', array('id' => $userid))) {
    // Do NOT print_error here as we are a document writer.
    die ('Invalid user ID');
}

$input = report_trainingsessions_batch_input($course);

// Security.

report_trainingsessions_back_office_access($course);

$config = get_config('report_trainingsessions');

// Get all results for this user.
$logs = use_stats_extract_logs($input->from, $input->to, $user->id, 0);
$aggregate = use_stats_aggregate_logs($logs, 'module', 0, $input->from, $input->to);

$fulltotal = 0;
list($displaycourses, $courseshort, $coursefull, $courseelapsed, $courseevents) = block_use_stats_get_coursetable($aggregate, $fulltotal, $fullevents);

// generate PDF
$pdf = new TCPDF('P', 'mm', 'A4', true, 'UTF-8', false);

$pdf->SetTitle(get_string('sessionreportdoctitle', 'report_trainingsessions'));
$pdf->setPrintHeader(false);
$pdf->setPrintFooter(false);
$pdf->SetAutoPageBreak(false, 0);
$pdf->setCellPaddings(2, 2, 2, 2);
$pdf->AddPage();

// Define variables.
// Portrait.
$x = 20;
$y = $config->pdfabsoluteverticaloffset;
$lineincr = 8;
$dblelineincr = $lineincr * 2;
$smalllineincr = 4;

// Set alpha to no-transparency.
// $pdf->SetAlpha(1);

// Add images and lines.
report_trainingsessions_draw_frame($pdf);

// Add images and lines.
report_trainingsessions_print_header($pdf);

// Add images and lines.
report_trainingsessions_print_footer($pdf);

// Prepare layout control.

$table = new html_table();
// head2 for printing will be prepared by report_trainingsessions_print_courseline_head.

$head2bgcolor = $config->head2bgcolor;
$head2txtcolor = $config->head2textcolor;
$head3bgcolor = $config->head3bgcolor;
$head3txtcolor = $config->head3textcolor;

if (empty($config->showhits)) {
    $table->pdfsize2 = array('25%', '50%', '25%');
    $table->pdfbgcolor2 = array($head2bgcolor, $head2bgcolor, $head2bgcolor, $head2bgcolor);
    $table->pdftxtcolor2 = array($head2txtcolor, $head2txtcolor, $head2txtcolor, $head2txtcolor);
    $table->pdfalign2 = array('L', 'L', 'R');
    $table->pdfalign3 = array('L', 'L', 'R');
    $table->pdfbgcolor3 = array($head3bgcolor, $head3bgcolor, $head3bgcolor, $head3bgcolor);
    $table->pdfcolor3 = array($head3txtcolor, $head3txtcolor, $head3txtcolor, $head3txtcolor);
    $table->pdfprintinfo = array(1,1,1);
} else {
    $table->pdfsize2 = array('20%', '40%', '30%', '10%');
    $table->pdfbgcolor2 = array($head2bgcolor, $head2bgcolor, $head2bgcolor);
    $table->pdftxtcolor2 = array($head2color, $head2color, $head2color, $head2color);
    $table->pdfalign2 = array('L', 'L', 'R', 'R');
    $table->pdfalign3 = array('L', 'L', 'R', 'R');
    $table->pdfbgcolor3 = array($head3bgcolor, $head3bgcolor, $head3bgcolor);
    $table->pdfcolor3 = array($head3txtcolor, $head3txtcolor, $head3txtcolor);
    $table->pdfprintinfo = array(1,1,1,1);
}

$y = report_trainingsessions_print_userinfo($pdf, $y, $user, $course, $input->from, $input->to, null, $config->recipient);

$y = report_trainingsessions_print_courseline_head($pdf, $y, $table);

foreach (array_keys($displaycourses) as $courseid) {
    $courseline = new StdClass;
    $courseline->idnumber = ''; // TODO fetch idnumber.
    $courseline->shortname = $courseshort[$courseid];
    $courseline->elapsed = $courseelapsed[$courseid];
    $courseline->events = $courseevents[$courseid];
    $y = report_trainingsessions_print_courseline($pdf, $y, $courseline, $table);
}

$summators = array('', '', report_trainingsessions_format_time($fulltotal, 'html'));
if (!empty($config->showhits)) {
    $summators[] = $fullevents;
}
$y = report_trainingsessions_print_sumline($pdf, $y, $summators, $table);
$y = report_trainingsessions_check_page_break($pdf, $y, $isnewpage, true, false);

report_trainingsessions_print_signboxes($pdf, $y + $lineincr, array('student' => true, 'authority' => true));

// Sending HTTP headers.
ob_end_clean();
$loadmode = (empty($filename)) ? 'S' : 'D';
$result = $pdf->Output($filename, $loadmode);
if ($loadmode == 'S') {
    echo $result;
}
exit(0);
// echo '200';